@@ -18,7 +18,7 @@
 package POE::Component::MessageQueue;
 use Moose;
 
-our $VERSION = '0.2.1';
+our $VERSION = '0.2.2';
 
 use POE 0.38;
 use POE::Component::Server::Stomp;
@@ -320,7 +320,7 @@ sub route_frame
 		},
 
 		SUBSCRIBE => sub {
-			my $ack_type = $frame->headers->{ack} || 'auto';
+			my $ack_type = $frame->headers->{ack};
 
 			$self->log('notice',
 				"RECV ($cid): SUBSCRIBE $destination_name (ack: $ack_type)");
@@ -328,7 +328,7 @@ sub route_frame
 			if(my $d = $self->get_destination ($destination_name) ||
 			           $self->make_destination($destination_name))
 			{
-				$client->subscribe($d, $ack_type);
+				$client->subscribe($d, $ack_type && $ack_type eq 'client');
 				$self->notify(subscribe => {destination => $d, client => $client});
 				$d->pump();
 			}
@@ -457,7 +457,7 @@ sub dispatch_message
 		if ($client->send_frame($msg->create_stomp_frame()))
 		{
 			$self->log(info => "Dispatching message $msg_id to client $client_id");
-			if ($subscriber->ack_type eq 'client')
+			if ($subscriber->client_ack)
 			{
 				$subscriber->ready(0);
 				$self->set_owner($msg_id => $subscriber);
