@@ -51,12 +51,12 @@ __PACKAGE__->meta->make_immutable();
 
 sub subscribe
 {
-	my ($self, $destination, $ack_type) = @_;
+	my ($self, $destination, $client_ack) = @_;
 
 	my $subscription = POE::Component::MessageQueue::Subscription->new(
 		destination => $destination,
 		client      => $self,
-		ack_type    => $ack_type,	
+		client_ack  => $client_ack,
 	);
 
 	$self->set_subscription($destination->name => $subscription);
